package com.sillycycle.bagleyd.life3d;

/*
 * @(#)Life3DDialog.java
 *
 * Structure from Java Sourcebook by Ed Anuff, pp 287-289
 *
 * Copyright 1993 - 2014  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.GridLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.sillycycle.bagleyd.util.ColumnLayout;
import com.sillycycle.bagleyd.util.Slider;

/**
 * The <code>Life3DDialog</code> class holds the configuration
 * setup for the <code>Life3DFrame</code>.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/life.html
 */

public class Life3DDialog extends Dialog {
	private static final long serialVersionUID = 42L;
	static boolean active = false;
	static Life3DFrame frame = null;

	static int columnsValue = Life3DInterface.DEFAULT_COLUMNS;
	static int statesValue = 2;
	static int neighborIndexValue = Life3DInterface.DEFAULT_NEIGHBOR_INDEX;
	static boolean randomOrientValue = true;
	static boolean lissajousValue = true;
	static boolean gridValue = false;
	static boolean wrapValue = true;
	static boolean visibleValue = true;
	static boolean monoValue = false;
	static boolean roundValue = false;
	static boolean wireframeValue = false;

	Slider columnsChanger = new Slider("Field Size",
		columnsValue, Life3DInterface.MIN_COLUMNS,
		Life3DInterface.MAX_COLUMNS);
	Life3DBoxes lifeBoxes = new Life3DBoxes();
	JCheckBox randomOrientCheckBox = new JCheckBox("Random Orient");
	JCheckBox lissajousCheckBox = new JCheckBox("Lissajous");
	JCheckBox gridCheckBox = new JCheckBox("Grid");
	JCheckBox wrapCheckBox = new JCheckBox("Wrap");
	JCheckBox visibleCheckBox = new JCheckBox("Visible");
	JCheckBox monoCheckBox = new JCheckBox("Mono");
	JCheckBox roundCheckBox = new JCheckBox("Round");
	JCheckBox wireframeCheckBox = new JCheckBox("Wireframe");
	JCheckBox statesCheckBox = new JCheckBox("4 state");

	public Life3DDialog(Life3DFrame frame, Frame parent,
			String titleText,
			String buttonText) {
		super(parent, titleText, true);

		JButton button = new JButton(buttonText);
		JPanel buttonPanel = new JPanel();
		JPanel sliderPanel = new JPanel();
		JPanel lifeBoxesPanel = new JPanel();
		JPanel togglePanel = new JPanel();

		columnsChanger.setWidth(150);
		buttonPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.CENTER));
		sliderPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		sliderPanel.add(columnsChanger, BorderLayout.PAGE_START);
		buttonPanel.add(sliderPanel, BorderLayout.PAGE_START);
		lifeBoxesPanel.add(lifeBoxes, BorderLayout.PAGE_START);
		buttonPanel.add(lifeBoxesPanel, BorderLayout.PAGE_START);
		togglePanel.setLayout(new GridLayout(3, 0));
		randomOrientCheckBox.setSelected(randomOrientValue);
		togglePanel.add(randomOrientCheckBox);
		lissajousCheckBox.setSelected(lissajousValue);
		togglePanel.add(lissajousCheckBox);
		gridCheckBox.setSelected(gridValue);
		togglePanel.add(gridCheckBox);
		wrapCheckBox.setSelected(wrapValue);
		togglePanel.add(wrapCheckBox);
		visibleCheckBox.setSelected(visibleValue);
		togglePanel.add(visibleCheckBox);
		monoCheckBox.setSelected(monoValue);
		togglePanel.add(monoCheckBox);
		roundCheckBox.setSelected(roundValue);
		togglePanel.add(roundCheckBox);
		wireframeCheckBox.setSelected(wireframeValue);
		togglePanel.add(wireframeCheckBox);
		statesCheckBox.setSelected(statesValue != 2);
		togglePanel.add(statesCheckBox);
		buttonPanel.add(togglePanel, BorderLayout.PAGE_START);

		buttonPanel.add(button, BorderLayout.PAGE_START);
		add(BorderLayout.PAGE_START, buttonPanel);
		pack();
		Life3DDialog.active = true;
		Life3DDialog.frame = frame;
		columnsChanger.addChangeListener(new ColumnsChangeListener());
		lifeBoxes.addItemListener(new LifeItemListener());
		lifeBoxes.addActionListener(new LifeActionListener());
		randomOrientCheckBox.addItemListener(new RandomOrientListener());
		lissajousCheckBox.addItemListener(new LissajousListener());
		gridCheckBox.addItemListener(new GridListener());
		wrapCheckBox.addItemListener(new WrapListener());
		visibleCheckBox.addItemListener(new VisibleListener());
		monoCheckBox.addItemListener(new MonoListener());
		roundCheckBox.addItemListener(new RoundListener());
		wireframeCheckBox.addItemListener(new WireframeListener());
		statesCheckBox.addItemListener(new StatesListener());

		button.addActionListener((ActionEvent event) -> {
			dispose();
			//else multiple Dialogs can be opened
			active = false;
		});

		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent event) {
				dispose();
				//else multiple Dialogs can be opened
				active = false;
			}
		});
	}

	class ColumnsChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = columnsValue;

			columnsValue = columnsChanger.getValue();
			if (oldValue != columnsValue) {
				frame.endGame();
				frame.setColumns(columnsValue);
				frame.startGame();
			}
		}
	}

	class LifeActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			int ni = neighborIndexValue;

			neighborIndexValue = lifeBoxes.getNeighborIndex();
//System.out.println("Life3DDialog.actionPerformed: " + ni + " " + neighborIndexValue);
			frame.setRules(ni, neighborIndexValue);
		}
	}

	class LifeItemListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			int ni = neighborIndexValue;

			neighborIndexValue = lifeBoxes.getNeighborIndex();
//System.out.println("Life3DDialog.itemStateChanged: " + ni + " " + neighborIndexValue);
			frame.setRules(ni, neighborIndexValue);
		}
	}

	class LissajousListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setLissajous(lissajousCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class RandomOrientListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setRandomOrient(randomOrientCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class GridListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setGrid(gridCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class WrapListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setWrap(wrapCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class VisibleListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setCellVisible(visibleCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class MonoListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setMono(monoCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class RoundListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setRound(roundCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class WireframeListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setWireframe(wireframeCheckBox.isSelected());
			frame.requestFocus();
		}
	}

	class StatesListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent e) {
			frame.setStates(statesCheckBox.isSelected() ? 4 : 2);
			frame.requestFocus();
		}
	}

	public void setColumns(int columnsValue) {
		Life3DDialog.columnsValue = columnsValue;
		columnsChanger.setValue(columnsValue);
	}

	public static int getColumns() {
		return columnsValue;
	}

	public void setStates(int statesValue) {
		// can only be 2 or 4
		if (statesValue == 2)
			Life3DDialog.statesValue = statesValue;
		else
			Life3DDialog.statesValue = 4;
		statesCheckBox.setSelected(statesValue != 2);
	}

	public static int getStates() {
		return (Life3DDialog.statesValue);
	}

	public void setNeighborIndex(int neighborIndexValue) {
		Life3DDialog.neighborIndexValue = neighborIndexValue;
		lifeBoxes.setNeighborIndex(neighborIndexValue);
	}

	public void setRuleIndex(Integer ruleIndex) {
		if (ruleIndex != null) {
			lifeBoxes.setRuleIndex(ruleIndex);
		}
	}

	public static int getNeighborIndex() {
		return neighborIndexValue;
	}

	public void setBirth(int value) {
		lifeBoxes.setBirth(value);
	}

	public void setSurvival(int value) {
		lifeBoxes.setSurvival(value);
	}

	public void setMenu(int neighborIndex, int birth, int survival) {
		lifeBoxes.setMenu(neighborIndex, birth, survival);
	}

	public int getBirth() {
		return lifeBoxes.getBirth();
	}

	public int getSurvival() {
		return lifeBoxes.getSurvival();
	}

	public void updateList() {
		lifeBoxes.updateList();
	}

	public void setRandomOrient(boolean flag) {
		Life3DDialog.randomOrientValue = flag;
		randomOrientCheckBox.setSelected(flag);
	}

	public static boolean getRandomOrient() {
		return randomOrientValue;
	}

	public void setLissajous(boolean flag) {
		Life3DDialog.lissajousValue = flag;
		lissajousCheckBox.setSelected(flag);
	}

	public static boolean getLissajous() {
		return lissajousValue;
	}

	public void setGrid(boolean flag) {
		Life3DDialog.gridValue = flag;
		gridCheckBox.setSelected(flag);
	}

	public static boolean getGrid() {
		return gridValue;
	}

	public void setWrap(boolean flag) {
		Life3DDialog.wrapValue = flag;
		wrapCheckBox.setSelected(flag);
	}

	public static boolean getWrap() {
		return wrapValue;
	}

	public void setCellVisible(boolean flag) {
		Life3DDialog.visibleValue = flag;
		visibleCheckBox.setSelected(flag);
	}

	public static boolean getCellVisible() {
		return visibleValue;
	}

	public void setMono(boolean flag) {
		Life3DDialog.monoValue = flag;
		monoCheckBox.setSelected(flag);
	}

	public static boolean getMono() {
		return monoValue;
	}

	public void setRound(boolean flag) {
		Life3DDialog.roundValue = flag;
		roundCheckBox.setSelected(flag);
	}

	public static boolean getRound() {
		return roundValue;
	}

	public void setWireframe(boolean flag) {
		Life3DDialog.wireframeValue = flag;
		wireframeCheckBox.setSelected(flag);
	}

	public static boolean getWireframe() {
		return wireframeValue;
	}

	public static boolean getActive() {
		return Life3DDialog.active;
	}
}
